//
//  SimpleTwitterClientViewController.m
//  SimpleTwitterClient
//
//  Created by Chris Adamson on 4/17/09.
//  Copyright Subsequently and Furthermore, Inc. 2009. All rights reserved.
//

#import "SimpleTwitterClientViewController.h"

//START:code.SimpleTwitterClientViewController.defineinterestingtagnames
#define INTERESTING_TAG_NAMES @"text", @"name", nil
//END:code.SimpleTwitterClientViewController.defineinterestingtagnames

/* for information on Twitter's XML API for the friends timeline, see
 http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses-public_timeline
 */


@implementation SimpleTwitterClientViewController

@synthesize tweetsView, activityIndicator, screenName;

- (void) testJSONOut {
	// Create new SBJSON parser object
	SBJSON *parser = [[SBJSON alloc] init];
	
	// Prepare URL request to download statuses from Twitter
	NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:@"http://twitter.com/statuses/public_timeline.json"]];
	
	// Perform request and get JSON back as a NSData object
	NSData *response = [NSURLConnection sendSynchronousRequest:request returningResponse:nil error:nil];
	
	// Get JSON as a NSString from NSData response
	NSString *json_string = [[NSString alloc] initWithData:response encoding:NSUTF8StringEncoding];
	
	// parse the JSON response into an object
	// Here we're using NSArray since we're parsing an array of JSON status objects
	NSArray *statuses = [parser objectWithString:json_string error:nil];
	
	// Each element in statuses is a single status
	// represented as a NSDictionary
	for(NSDictionary *status in statuses)
	{
		// You can retrieve individual values using objectForKey on the status NSDictionary
		// This will print the tweet and username to the console
		NSLog(@"%@ - %@", [status objectForKey:@"text"], [[status objectForKey:@"user"] objectForKey:@"screen_name"]);
	}
}


//START:code.SimpleTwitterClientViewController.updateTweets
-(IBAction) updateTweets {
	tweetsView.text = @"";
	[tweetsData release];
	tweetsData = [[NSMutableData alloc] init];
	
	NSString* twitterUserTimelineUrl = @"http://twitter.com/statuses/user_timeline/";

//	NSString* twitterUrl = [[twitterUserTimeline 
//						  stringByAppendingString:screenName.text] stringByAppendingString:@".xml"];
	NSString* twitterUrl = [NSString stringWithFormat:@"%@%@%@", twitterUserTimelineUrl, screenName.text, @".xml"];
	
	NSURL *url = [NSURL URLWithString:twitterUrl];
	
	NSURLRequest *request = [[NSURLRequest alloc] initWithURL: url];	
	NSURLConnection *connection = [[NSURLConnection alloc]
			   initWithRequest:request
			   delegate:self];
	[connection release];
	[request release];
	[activityIndicator startAnimating];
	[self testJSONOut];
}


//END:code.SimpleTwitterClientViewController.updateTweets

//START:code.SimpleTwitterClientViewController.startparsingtweets
- (void) startParsingTweets {
	NSXMLParser *tweetParser = [[NSXMLParser alloc] initWithData:tweetsData];
	tweetParser.delegate = self;
	[tweetParser parse];
	[tweetParser release];
}
//END:code.SimpleTwitterClientViewController.startparsingtweets

#pragma mark UIViewController methods

//START:code.SimpleTwitterClientViewController.viewdidload
- (void) viewDidLoad {
	[super viewDidLoad];
	interestingTags = [[NSSet alloc] initWithObjects: INTERESTING_TAG_NAMES];
}
//END:code.SimpleTwitterClientViewController.viewdidload

#pragma mark NSURLConnection callbacks

//START:code.SimpleTwitterClientViewController.connectiondidreceivedata
- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[tweetsData appendData: data];
}
//END:code.SimpleTwitterClientViewController.connectiondidreceivedata

-(void) connection:(NSURLConnection *)connection
  didFailWithError: (NSError *)error {
	UIAlertView *errorAlert = [[UIAlertView alloc]
				   initWithTitle: [error localizedDescription]
				   message: [error localizedFailureReason]
				   delegate:nil
				   cancelButtonTitle:@"OK"
				   otherButtonTitles:nil];
	[errorAlert show];
	[errorAlert release];
	[activityIndicator stopAnimating];
}

//START:code.SimpleTwitterClientViewController.connectiondidfinishloading
- (void) connectionDidFinishLoading: (NSURLConnection*) connection {
	[activityIndicator stopAnimating];
	[self startParsingTweets];
}
//END:code.SimpleTwitterClientViewController.connectiondidfinishloading



#pragma mark NSXMLParser callbacks

//START:code.SimpleTwitterClientViewController.parserdidstartdocument
- (void)parserDidStartDocument:(NSXMLParser *)parser {
	[tweetsString release];
	tweetsString = [[NSMutableString alloc]
			initWithCapacity: (20 * (140 + 20)) ];
	currentElementName = nil;
	currentText = nil;
}
//END:code.SimpleTwitterClientViewController.parserdidstartdocument


//START:code.SimpleTwitterClientViewController.parserdidstartelement
- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName
		namespaceURI:(NSString *)namespaceURI
		qualifiedName:(NSString *)qualifiedName
		attributes:(NSDictionary *)attributeDict {
	if ([elementName isEqualToString:@"status"]) {
		[currentTweetDict release];
		currentTweetDict = [[NSMutableDictionary alloc]
			initWithCapacity: [interestingTags count]];
	}
	else if ([interestingTags containsObject: elementName]) { //<label id="code.SimpleTwitterClient.SimpleTwitterClientViewController.didstartelement.setcontainsobject"/>
		currentElementName = elementName;
		[currentText release];
		currentText = [[NSMutableString alloc] init];
	}
 }
//END:code.SimpleTwitterClientViewController.parserdidstartelement


//START:code.SimpleTwitterClientViewController.parserdidendelement
- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName
		namespaceURI:(NSString *)namespaceURI
		qualifiedName:(NSString *)qName {
	if ([elementName isEqualToString:currentElementName]) {
		[currentTweetDict setValue: currentText forKey: currentElementName];
		currentText = nil;
	} else if ([elementName isEqualToString:@"status"]) {
		[tweetsString appendFormat:@"%@: %@\n\n",
			[currentTweetDict valueForKey:@"name"],
			[currentTweetDict valueForKey:@"text"]];
	}
}
//END:code.SimpleTwitterClientViewController.parserdidendelement

//START:code.SimpleTwitterClientViewController.parserfoundcharacters
- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
	[currentText appendString:string];
}
//END:code.SimpleTwitterClientViewController.parserfoundcharacters

//START:code.SimpleTwitterClientViewController.parserdidenddocument
- (void)parserDidEndDocument:(NSXMLParser *)parser {
	tweetsView.text = tweetsString;
}
//END:code.SimpleTwitterClientViewController.parserdidenddocument

// TODO: add http authentication for friends timeline

#pragma mark -
#pragma mark ViewController boilerplate


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
    // Release anything that's not essential, such as cached data
}


- (void)dealloc {
    [super dealloc];
	[tweetsView release];
	[activityIndicator release];
	[interestingTags release];
}


@end
